@@ -3,8 +3,6 @@ package App::Parcimonie;
 use warnings;
 use strict;
 
-our $VERSION = '0.7.1'; # VERSION
-
 use App::Parcimonie::GnuPG::Interface;
 use Carp;
 use Clone qw{clone};
@@ -12,14 +10,19 @@ use Config::General qw/ParseConfig/;
 use Encode;
 use English qw/-no_match_vars/;
 use File::HomeDir;
+use IPC::System::Simple qw/capturex/;
 use List::MoreUtils qw/uniq/;
 use List::Util qw/shuffle/;
-use Path::Class;
+use Path::Tiny;
 use Try::Tiny;
+use version 0.77;
 
 use Exporter;
 our @ISA = qw(Exporter);
 
+use namespace::clean;
+
+
 =head1 NAME
 
 App::Parcimonie - tools for the parcimonie program
@@ -32,7 +35,7 @@ See bin/parcimonie :)
 
 =cut
 
-our @EXPORT = qw/pickRandomItems gpgPublicKeys gpgRecvKeys
+our @EXPORT = qw/pickRandomItems gpgPublicKeys gpgRecvKeys gpg_is_v21
                  checkGpgHasDefinedKeyserver averageLapseTime/;
 
 my $codeset;
@@ -67,6 +70,17 @@ sub pickRandomItems {
     return @randomized_pool[0..$n - 1];
 }
 
+=head2 gpg_is_v21()
+
+Detects the version of the gpg binary in the $PATH.
+Returns true iff. it is 2.1 or newer.
+
+=cut
+sub gpg_is_v21 {
+    version->parse(App::Parcimonie::GnuPG::Interface->new()->version())
+        >= version->parse('2.1.0');
+}
+
 =head2 gpgPublicKeys()
 
 Returns the list of key fingerprints found in the public keyring.
@@ -85,7 +99,7 @@ sub gpgPublicKeys {
     $gnupg->get_public_keys_hex();
 }
 
-=head2 gpgRecvKeys( @keyids )
+=head2 gpgRecvKeys( $keyids, $gnupg_options )
 
 Imports the keys with the given key IDs from a keyserver.
 
@@ -148,18 +162,35 @@ sub gpgRecvKeys {
 
 Throws an exception if no keyserver is defined in GnuPG configuration.
 
+When using GnuPG 2.x, a keyserver can be defined either in dirmngr's
+configuration or in good old gpg.conf, so we check both.
+
 =cut
 
 sub checkGpgHasDefinedKeyserver {
-    my $arg_ref = shift;
-    my $gnupg_homedir = $arg_ref->{homedir};
+    my $gnupg_options = shift;
+    my $arg_ref       = shift;
+    my $gnupg_homedir = $gnupg_options->{homedir};
+    my $gnupg2        = $arg_ref->{gnupg2};
+
+    if ($gnupg2) {
+        my @output = capturex(qw{gpg-connect-agent --dirmngr keyserver /bye});
+        my $res = pop @output;
+        $res eq "OK\n" || croak "Agent replied: $res";
+        if (@output) {
+            my $last_keyserver = pop @output;
+            if ($last_keyserver =~ m{\A [S] \s+ KEYSERVER \s+}xms) {
+                return;
+            }
+        }
+    }
 
     my $gpgconf;
     if (defined $gnupg_homedir) {
-        $gpgconf = file($gnupg_homedir, 'gpg.conf');
+        $gpgconf = path($gnupg_homedir, 'gpg.conf');
     }
     else {
-        $gpgconf = file(File::HomeDir->my_home, '.gnupg', 'gpg.conf');
+        $gpgconf = path(File::HomeDir->my_home, '.gnupg', 'gpg.conf');
     }
 
     -f $gpgconf
