@@ -14,14 +14,10 @@ use MooX::late;
 
 use 5.10.1;
 use App::Parcimonie;
-use App::Parcimonie::DBus::Object;
+use Carp;
 use Encode;
 use English qw{-no_match_vars};
-use List::MoreUtils qw{any}; # XXX: Perl 5.20: use the version from List::Util
-use Net::DBus;
-use Net::DBus::Reactor;
-use Net::DBus::Service;
-use Net::DBus::Test::MockObject;
+use List::Util qw{any};
 use Time::Duration::Parse qw(parse_duration);
 use Try::Tiny;
 use Type::Utils qw{declare as where coerce from};
@@ -49,6 +45,11 @@ option 'verbose'  => (
     isa => 'Bool', is => 'ro', default => 0
 );
 
+option 'with_dbus'  => (
+    documentation => q{Send activity updates on the org.parcimonie.daemon D-Bus service},
+    isa => 'Bool', is => 'ro', default => 0
+);
+
 option 'gnupg_homedir' => (
     documentation => q{GnuPG homedir.},
     isa           => Dir,
@@ -66,6 +67,14 @@ option 'gnupg_extra_args' => (
     format        => 's@',
 );
 
+has 'gnupg_builtin_args' => (
+    isa           => 'ArrayRef[Str]',
+    is            => 'ro',
+    default       => sub { [
+        '--import-options' => 'merge-only',
+    ] },
+);
+
 has 'gnupg_options' => (
     isa           => 'HashRef',
     is            => 'ro',
@@ -98,20 +107,8 @@ has 'iterate_id' => (
 
 has 'dbus_object' => (
     isa           => 'Object',
-    is            => 'rw',
-    default       => sub {
-        my ($bus, $service);
-        if ($ENV{HARNESS_ACTIVE}) {
-            $bus = Net::DBus->test;
-            $service = $bus->export_service("org.parcimonie.daemon");
-            Net::DBus::Test::MockObject->new($service, "/org/parcimonie/daemon/object");
-        }
-        else {
-            $bus = Net::DBus->session;
-            $service = $bus->export_service("org.parcimonie.daemon");
-            App::Parcimonie::DBus::Object->new($service);
-        }
-    },
+    is            => 'ro',
+    lazy_build    => 1,
 );
 
 option 'gnupg_already_torified' => (
@@ -122,6 +119,20 @@ option 'gnupg_already_torified' => (
     default   => sub { 0; },
 );
 
+option 'gnupg2' => (
+    documentation => q{Use GnuPG 2.1+ (default: autodetect gpg's version).},
+    isa      => 'Bool',
+    is       => 'ro',
+    required => 0,
+    default  => sub { gpg_is_v21(); },
+);
+
+has 'memory_usage' => (
+    isa       => 'Memory::Usage',
+    is        => 'ro',
+    required  => 0,
+);
+
 
 =head1 METHODS
 
@@ -135,8 +146,44 @@ Post-constructor.
 sub BUILD {
     my $self = shift;
 
+    $self->record_memory_usage("[BUILD]: entering");
+
     $self->keyserver_defined_on_command_line
-        or checkGpgHasDefinedKeyserver($self->gnupg_options);
+        or checkGpgHasDefinedKeyserver(
+            $self->gnupg_options,
+            {
+                gnupg2 => $self->gnupg2,
+            });
+
+    if ($self->with_dbus) {
+        $self->record_memory_usage("[BUILD]: starting to load D-Bus dependencies");
+        require App::Parcimonie::DBus::Object;
+        require Net::DBus;
+        require Net::DBus::Reactor;
+        require Net::DBus::Service;
+        require Net::DBus::Test::MockObject;
+        $self->record_memory_usage("[BUILD]: finished loading D-Bus dependencies");
+    }
+
+    $self->record_memory_usage("[BUILD]: leaving");
+}
+
+sub _build_dbus_object {
+    my $self = shift;
+
+    croak "D-Bus is disabled." unless $self->with_dbus;
+
+    my ($bus, $service);
+    if ($ENV{HARNESS_ACTIVE}) {
+        $bus = Net::DBus->test;
+        $service = $bus->export_service("org.parcimonie.daemon");
+        Net::DBus::Test::MockObject->new($service, "/org/parcimonie/daemon/object");
+    }
+    else {
+        $bus = Net::DBus->session;
+        $service = $bus->export_service("org.parcimonie.daemon");
+        App::Parcimonie::DBus::Object->new($service);
+    }
 }
 
 =head2 run
@@ -149,6 +196,37 @@ sub run {
     my $opt  = shift;
     my $args = shift;
 
+    $self->record_memory_usage("[run]: entering");
+
+    if ($self->with_dbus) {
+        $self->run_with_dbus();
+    }
+    else {
+        use JSON::PP;
+        $self->run_without_dbus();
+    }
+}
+
+sub run_without_dbus {
+    my $self = shift;
+
+    while (1) {
+        my $next_sleep_time = $self->iterate;
+        say encode_json({
+            state   => 'Sleeping',
+            details => {
+                duration => $next_sleep_time,
+            },
+        });
+        sleep($next_sleep_time);
+    }
+}
+
+sub run_with_dbus {
+    my $self = shift;
+
+    $self->record_memory_usage("[run_with_dbus]: entering");
+
     my $reactor = Net::DBus::Reactor->main();
 
     my $initial_sleep_time = 1 * 1000;
@@ -166,6 +244,10 @@ sub run {
                         "Will now sleep %i seconds.",
                         $next_sleep_time
                     ));
+                    $self->notify({
+                        signal => 'Sleeping',
+                        duration => $next_sleep_time,
+                    });
                     $reactor->toggle_timeout(
                         $self->iterate_id,
                         1,
@@ -195,9 +277,14 @@ sub fatal {
 
 sub _build_gnupg_options {
     my $self = shift;
-    my %opts;
-    $opts{homedir}    = $self->gnupg_homedir    if defined $self->gnupg_homedir;
-    $opts{extra_args} = $self->gnupg_extra_args if defined $self->gnupg_extra_args;
+    my %opts = (
+        extra_args => [],
+    );
+    $opts{homedir} = $self->gnupg_homedir if defined $self->gnupg_homedir;
+    push @{$opts{extra_args}}, @{$self->gnupg_builtin_args}
+        if defined $self->gnupg_builtin_args;
+    push @{$opts{extra_args}}, @{$self->gnupg_extra_args}
+        if defined $self->gnupg_extra_args;
     return \%opts;
 }
 
@@ -218,6 +305,38 @@ sub keyserver_defined_on_command_line {
     } @{$self->gnupg_extra_args};
 }
 
+sub notify {
+    my $self = shift;
+    my $args = shift;
+
+    if ($self->with_dbus) {
+        my $signal = $args->{signal};
+        my @args_name;
+        if ($signal eq 'FetchBegin') {
+            @args_name = qw{keyid};
+        }
+        elsif ($signal eq 'FetchEnd') {
+            @args_name = qw{keyid success gpg_error};
+        }
+        elsif ($signal eq 'Sleeping') {
+            @args_name = qw{duration};
+        }
+        else {
+            croak "Unsupported signal: $signal";
+        }
+
+        my @args;
+        push @args, $args->{$_} for (@args_name);
+        $self->dbus_object->emit_signal($signal, @args);
+    }
+    else {
+        say encode_json {
+            state   => $args->{signal},
+            details => $args,
+        };
+    }
+}
+
 sub tryRecvKey {
     my $self  = shift;
     my $keyid = shift;
@@ -225,13 +344,14 @@ sub tryRecvKey {
     my $gpg_error;
 
     $self->debug(sprintf("tryRecvKey: trying to fetch %s", $keyid));
-    $self->dbus_object->emit_signal("FetchBegin", $keyid);
+    $self->notify({ signal => 'FetchBegin', keyid => $keyid });
 
     try {
         $gpg_output = gpgRecvKeys(
             [ $keyid ],
             $self->gnupg_options,
             already_torified => $self->gnupg_already_torified,
+            gnupg2           => $self->gnupg2,
         );
     } catch {
         $gpg_error = $_;
@@ -248,8 +368,12 @@ sub tryRecvKey {
         $gpg_error = '';
     }
 
-    $self->dbus_object->emit_signal("FetchEnd", $keyid, $success, $gpg_error);
-
+    $self->notify({
+        signal    => 'FetchEnd',
+        keyid     => $keyid,
+        success   => $success,
+        gpg_error => $gpg_error,
+    });
 }
 
 sub next_sleep_time {
@@ -284,9 +408,13 @@ Returns next sleep time (seconds).
 sub iterate {
     my $self = shift;
     my $default_sleep_time = 600;
+
+    $self->record_memory_usage("[iterate]: entering");
+
     my @public_keys = gpgPublicKeys(
         $self->gnupg_options,
         already_torified => $self->gnupg_already_torified,
+        gnupg2           => $self->gnupg2,
     );
     unless (@public_keys) {
         warn "No public key was found.";
@@ -297,10 +425,30 @@ sub iterate {
     # allow the GC to free some memory
     undef(@public_keys);
 
+    $self->record_memory_usage("[iterate]: before fetching key $keyid");
     $self->tryRecvKey($keyid);
+    $self->record_memory_usage("[iterate]: after fetching key $keyid");
 
     return $next_sleep_time;
 }
 
+=head2 record_memory_usage
+
+Args: a message (Str) indicating when we're run.
+Records memory usage in the memory_usage attribute,
+iff. the REPORT_MEMORY_USAGE environment variable is set to a true value.
+
+=cut
+sub record_memory_usage {
+    my $self = shift;
+    my $message = shift;
+
+    return unless exists $ENV{REPORT_MEMORY_USAGE}
+        && defined $ENV{REPORT_MEMORY_USAGE}
+        && $ENV{REPORT_MEMORY_USAGE};
+
+    $self->memory_usage->record($message);
+}
+
 no Moo;
 1; # End of App::Parcimonie::Daemon
