@@ -9,14 +9,16 @@ Have a look to App::Parcimonie::Daemon for a full-blown real life usage example.
 =cut
 
 package App::Parcimonie::GnuPG::Interface;
-use Any::Moose;
-extends 'GnuPG::Interface';
 
-our $VERSION = '0.7.1'; # VERSION
-
-use namespace::autoclean;
+use Moo;
+use MooX::late;
 use Carp;
 use File::Which qw{which};
+use IPC::System::Simple qw{system systemx};
+
+use namespace::clean;
+
+extends 'GnuPG::Interface';
 
 has 'already_torified' => (
     isa      => 'Bool',
@@ -25,13 +27,31 @@ has 'already_torified' => (
     default  => sub { 0; },
 );
 
+has 'gnupg2' => (
+    isa      => 'Bool',
+    is       => 'rw',
+    required => 0,
+    default  => sub { 0; },
+);
+
 after 'BUILD' => sub {
     my $self = shift;
-    unless ($self->already_torified) {
-        foreach my $prog (qw{torsocks parcimonie-torified-gpg}) {
-            defined which($prog) or croak "$prog not found in \$PATH";
+    if ($self->gnupg2) {
+        unless ($self->already_torified) {
+            system(q{echo 'use-tor:0:1' | gpgconf --change-options dirmngr});
+            # Passing --runtime to the previous command does not work,
+            # so we have to:
+            systemx(qw{gpgconf --reload dirmngr});
+        }
+        $self->call('gpg2');
+    }
+    else {
+        unless ($self->already_torified) {
+            foreach my $prog (qw{torsocks parcimonie-torified-gpg}) {
+                defined which($prog) or croak "$prog not found in \$PATH";
+            }
+            $self->call('parcimonie-torified-gpg');
         }
-        $self->call('parcimonie-torified-gpg');
     }
 };
 
@@ -94,5 +114,5 @@ sub get_public_keys_hex {
 
 }
 
-no Any::Moose;
+no Moo;
 1; # End of App::Parcimonie::GnuPG::Interface
